(* module Assem = struct *)

open Absyntax
(* fonctions d'utilite *)

let implode cl =
    let n = List.length cl
    in let s = String.create n
       in for i = 0 to n-1
          do String.set s i (List.nth cl i)
          done; s

let explode s = 
  let n = String.length s
  and l = ref []
  in for i = n-1 downto 0 do
     l:= (String.get s i):: !l 
     done; !l

(* le module assembleur *)


  type reg = string
  type temp = Temp.temp
  type label = Temp.label

    type instr = OPER of roper 
             | LABEL of rlabel
             | MOVE of rmove
             | COMMENT of string

  and  roper = {o_assem: string; o_dst: temp list; o_src: temp list; jump: label list option}
  and  rlabel = {l_assem: string; lab: label}
  and  rmove = {m_assem: string; m_dst: temp; m_src: temp}

  let format saytemp =
    let speak(assem,dst,(src:temp list),jump) =
	    let saylab = Symbol.name    
	    in let rec f = function
                    ('`':: 's':: i::rest) -> 
		    (explode(saytemp(List.nth src (Char.code i - Char.code '0'))) @ f rest)
		  | ( '`':: 'd':: i:: rest) -> 
		    (explode(saytemp(List.nth dst (Char.code i - Char.code '0'))) @ f rest)
		  | ( '`':: 'j':: i:: rest) ->
		    (explode(saylab(List.nth jump (Char.code i - Char.code '0'))) @ f rest)
		  | ( '`':: '`':: rest) -> '`' :: f rest
		  | ( '`':: _ :: rest) -> failwith "bad Assem format"
		  | (c :: rest) -> (c :: f rest)
		  | [] -> []
	    in implode(f(explode assem))
      in function
            OPER{o_assem=assem;o_dst=dst;o_src=src;jump=Some l} -> speak(assem,dst,src,l)
          | OPER{o_assem=assem;o_dst=dst;o_src=src;jump=None} -> speak(assem,dst,src,[])
	  | LABEL{l_assem=assem} -> assem
	  | COMMENT s -> s
	  | MOVE{m_assem=assem;m_dst=dst;m_src=src} -> speak(assem,[dst],[src],[])

(* Exemples:

let t1 = OPER {o_assem="LI `d0,2(`s0)"; 
              o_dst=[Temp.new_temp()];
              o_src=[Temp.new_temp()];
              jump=None};;        

let t2 = OPER {o_assem="JAL `j0"; 
              o_dst=[Temp.new_temp()];
              o_src=[];
              jump=Some [Symbol.symbol "endloop"]};;        

format (fun i -> "t"^(string_of_int i)) t1;;    
format (fun i -> "t"^(string_of_int i)) t2;;    

*)

(* end *)
