open Absyntax
open Prabsyn

let predefuns = (Symbol.addlist [(Symbol.symbol "printint"),0;
                                  (Symbol.symbol "ord"),0;
                                  (Symbol.symbol "chr"),0;
                                  (Symbol.symbol "print"),0;
                                  (Symbol.symbol "concat"),0;
                                  (Symbol.symbol "getchar"),0;
                                ] 
                                (Symbol.mkempty())) 

let predeframes = (Symbol.addlist [(Symbol.symbol "printint"),(Symbol.symbol "printint");
                                  (Symbol.symbol "ord"),(Symbol.symbol "ord");
                                  (Symbol.symbol "chr"),(Symbol.symbol "chr");
                                  (Symbol.symbol "print"),(Symbol.symbol "print");
                                  (Symbol.symbol "concat"),(Symbol.symbol "concat");
                                  (Symbol.symbol "getchar"),(Symbol.symbol "getchar");
                                ] 
                                (Symbol.mkempty())) 

let level_main = 1;; (* level du main *)

exception Erreur_de_syntaxe of int;;

(* compute AST *)
let ast fn = let lexbuf = Lexing.from_channel (open_in fn) in
    try
      Parser.programme Lexer.token lexbuf
    with Parsing.Parse_error -> raise (Erreur_de_syntaxe (Lexing.lexeme_start lexbuf));;

(* Static analysis: typing the AST *)

let typeast ast = ();;

(* compute level and offset attributes AST *)

let attrast ast = ast ;;

(* convert tree to list of fragments of intermediate representation *)

let ir ast = [];;

let print_frame _ = ();;

(* convert fragments to linearized code *)

let stms frags = [];;

let print_ir _ = ();;

(* convert to assembler *)

let noalloc _ = ();;

let simplealloc _ = ();;

let instrs allocstrategy _ = [];;

let print_rawasm _ = ();;

let print_asm _ = ();;

let main() = 
  let past=ref false and tast=ref false and pattr=ref false and pint=ref false and pfrags=ref false and prasm=ref false and pasm=ref false in
  let doasource fn =
    if !past then (Printf.printf "\n-------AST-------\n\n";PrintAbsyn.print (stdout,ast fn));
    if !tast then (Printf.printf "\n-------TAST------\n\n"; let _ = typeast (ast fn) in ());
    if !pattr then (Printf.printf "\n-------ATTRAST--\n\n";PrintAbsyn.print (stdout,attrast (ast fn)));
    if !pint then (Printf.printf "\n-------IR--------\n\n";List.iter print_frame (ir (attrast (ast fn))));
    if !pfrags then (Printf.printf "\n-------FRAGS-----\n\n";print_ir (stms (ir (attrast (ast fn)))));
    if !prasm then (Printf.printf "\n-------AASSEM----\n\n";print_rawasm (instrs noalloc (stms (ir (attrast (ast fn))))));
    if !pasm then (Printf.printf "\n-------ASSEM-----\n\n";print_asm (instrs simplealloc (stms (ir (attrast (ast fn))))))
  in 
  Arg.parse
    [("-ast",Arg.Set past,"  \tPrint abstract syntax tree");
     ("-tast",Arg.Set tast,"  \tPrint typed abstract syntax tree");
     ("-lo",Arg.Set pattr,"  \tPrint attributed abstract syntax tree");
     ("-int",Arg.Set pint,"  \tPrint intermediate representation fragments");
     ("-lin",Arg.Set pfrags,"  \tPrint linearized, canonical intermediate representation fragments");
     ("-rawasm",Arg.Set prasm,"  \tPrint abstract assembler with no register allocation");
     ("-asm",Arg.Set pasm,"  \tPrint assembler with register allocation")]
    (fun fn -> doasource fn) "Usage: ctrigre [-ast] [-tast] [-lo] [-int] [-lin] [-rawasm] [-asm] file ... file"
in Printexc.catch main ()
;;
