	# Assembler runtime support for CTigre
	# CTigre wrappers for the SPIM bare OS functions, as
	# necessary to implement the predefined library functions in CTigre
	# (C) Roberto Di Cosmo 2003

	# The structure of the stack after a CTigre call is:	
	#
	#             
	#             parameter n
	#	      ...
	#             parameter 1
	#     SP ->   static link
	#             saved FP
	#             saved RA
	#	      ...

	.text	

	# Output

	# printint
printint:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp	
	sub $sp $sp 12
	
	lw $a0 4($fp)	# find parameter passed on the stack (integer)
	li $v0 1	# load system call code for print_int
	# here we (may) save the return address in the place where the parameter was passed
	syscall         # call the OS
	
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

	
	# printfloat
printfloat:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp

	sub $sp $sp 12	
	l.s $f12 4($fp)	# find parameter passed on the stack (single precision floating point number)
	li $v0 2	# load system call code for print_float
	# here we (may) save the return address in the place where the parameter was passed
	syscall         # call the OS

	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

	
	# print
print:	
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12	
	
	lw $a0 4($fp)	# find parameter passed on the stack (zero terminated string)
	li $v0 4	# load system call code for print_string
	syscall		# call the OS
	
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function


	# Input
	
	# getstring
getstring:	
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12

	# allocate a chunk of memory to hold the string buffer
	lw  $a1 4($fp)	# length we have been passed
	add $a1 $a1 1   # add one for 0 termination
	sw $a1 0($sp)
	sub $sp $sp 4
	jal malloc      # call malloc
	add $sp $sp 4
	
	add $a0 $0 $3	# store the address of the allocated string buffer in $a0
	lw  $a1 4($fp)	# we read a string of length up to the parameter we have been passed
	add $a1 $a1 1   # add one for 0 termination
	li  $v0 8
	sw  $3 0($sp)   # save string pointer
	sub $sp $sp 4
	syscall		# call the OS
	add $sp $sp 4
	lw $3 0($sp)    # result register contains string pointer
	
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function


	# getchar
getchar:	
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12	

	add $a0 $0 $29	# store the one-char string in first free cell on the stack
	li $a1 2	# we read a string of length 1, plus one byte for zero termination
	li $v0 8
	syscall		# call the OS
	lbu $3 0($29)   # get the byte (unsigned) on the stack and put it into the result register as a 4 word quantity

	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

		
	# readint
readint:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12	

	li $v0 5	# load system call code for read_int
	syscall         # call the OS
	add $3 $v0 $0   # copy result in the result register
	
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

	
	# readfloat
readfloat:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12	

	li $v0 6	# load system call code for read_float
	syscall         # call the OS
	s.s $f12 0($29) # copy result from $f12 to the result register 
	lw $3 0($29)    # through the stack
	
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

	
	# malloc
malloc:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12
	lw $a0 4($fp)	# find parameter passed on the stack (amount of memory)

	add $a0 $a0 4   # add 4 to make sure we are not zeroed out by the stupid SPIM sbkr bug
	                # (SPIM zeores out on 4 byte boundaries!!!!)
	
	li $v0 9	# load system call code for sbrk
	syscall		# call the OS, on return address of block is in $v0
	add $3 $v0 $0   # copy it in the result register

	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function

	# Conversions

	# ord and char are the id function, due to the (expensive) coding of chars as integers
chr:
ord:	

	lw $3 4($29)	# find parameter passed on the stack (integer), and just return it
	jr $ra		# return to CTigre calling function
	
	# String operations
	
	# size 
size:	
	lw $a0 4($29)	# find parameter passed on the stack (zero terminated string)
	li $3 0	        # $3 will contain the length
__loop:	lbu $2 0($a0)   # load the next byte in the string in $4
	beqz $2 __end   # end if zero 
	add $a0 $a0 1	# 
	add $3  $3 1	#
	j __loop	
__end:	jr $ra		# return to CTigre calling function

	# make a string from a char
mkstring:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 12
        li $3 2
	sw $3 0($sp)
	sub $sp $sp 4
	jal malloc
	add $sp $sp 4
	lw $11 4($fp)
	sb $11 0($3)
	sb $0  1($3)
	move $sp $fp
	lw $31 -8($sp)
	lw $fp -4($sp)
	jr $ra		# return to CTigre calling function
	
concat:
	sw $fp -4($sp)
	sw $31 -8($sp)
	move $fp $sp
	sub $sp $sp 24
	lw $11  8($fp)	# find second parameter passed on the stack (pointer to second string)
	sw $11  0($29)
	sub $sp $sp 4
	jal size
	add $sp $sp 4
	sw  $3 -12($fp) # in -12($fp), size of second string
	lw $11  4($fp)	# find first parameter passed on the stack (pointer to first string)
        sw $11  0($29)
	sub $sp $sp 4
	jal size
	add $sp $sp 4
	sw $3  -16($fp) # in -16($fp), size of first string
	lw $11 -12($fp) # compute size of new string
	add $11 $3 $11
	sw $11 -20($fp) # -20($fp) contains new size without zero
	add $11 $11 1   # plus one for the terminating zero
	sw $11 0($sp)   # call malloc
	sub $sp $sp 4
	jal malloc
	add $sp $sp 4
	                # now do the copy	
	add $11 $3 0    # in $3, base address of new string
	lw $a0 4($fp)   # address of first string
	lw $a1 -16($fp) # size of first string
__loop01:	
	beqz $a1, __end01
	lbu  $2, 0($a0) # load char
	sb   $2, 0($11) # store char
	add  $11 $11 1
	add  $a0 $a0 1
	sub  $a1 $a1 1
	j __loop01
__end01:
	lw $a0 8($fp)   # address of second string
	lw $a1 -12($fp) # size of second string
__loop02:	
	beqz $a1, __end02
	lbu  $2, 0($a0) # load char
	sb   $2, 0($11) # store char
	add  $11 $11 1
	add  $a0 $a0 1
	sub  $a1 $a1 1
	j __loop02
__end02:
	sb $0 0($11)    # terminate new string
        move $sp $fp
	lw $fp -4($sp)		
	lw $31 -8($sp)
	jr $ra		# return to CTigre calling function

#substring:		

